#include "X3D_CoordMergerRenderingVisitor.h"
#include "X3D_Coordinate.h"
#include "X3D_IndexedFaceSet.h"

#include <iostream>
#include <vector>

using namespace std;

namespace X3DTK {
namespace X3D {

CoordMergerRenderingVisitor::CoordMergerRenderingVisitor()
: RenderingVisitor()
{
  // Enter functions.
  define(Recorder<Coordinate>::getEnterFunction(&CoordMergerRenderingVisitor::enterCoordinate));
}

void CoordMergerRenderingVisitor::enterCoordinate(Coordinate *C) 
{
  CoordMergerStateVariables *stateVariables = Singleton<CoordMergerStateVariables>::getInstance();
  if (!stateVariables->getNode(C))
  {
    // Computing the new point array.
  
    MFVec3f newPoint;
    const MFVec3f &point = C->getPoint();
    vector<unsigned int> newIndex(point.size());
    vector<unsigned int>::iterator itNewIndex = newIndex.begin();
  
    for (MFVec3f::const_iterator itPoint = point.begin(); itPoint != point.end(); ++itPoint)
    {
      bool found = false;
      for (unsigned int nnew = 0; nnew < newPoint.size(); ++nnew)
      {
        if (*itPoint == newPoint[nnew])
        {
          *itNewIndex = nnew;
          found = true;
          break;
        }
      }
      if (!found)
      {
        *itNewIndex = newPoint.size();
        newPoint.push_back(*itPoint);
      }  
      ++itNewIndex;
    }
    
    C->setPoint(newPoint);
    
    // Computing the new IndexedFaceSet.
    
    MFAbstractNode parentList = C->getParentList();
    for (MFAbstractNode::const_iterator it = parentList.begin(); it != parentList.end(); ++it)
    {
      if (dynamic_cast<IndexedFaceSet *>(*it) != 0) 
      {
        MFInt32 newCoordIndex = static_cast<IndexedFaceSet *>(*it)->getCoordIndex();
        for (MFInt32::iterator itCoordIndex = newCoordIndex.begin(); itCoordIndex != newCoordIndex.end(); ++itCoordIndex)
        {
          if (*itCoordIndex != -1)
            *itCoordIndex = newIndex[*itCoordIndex];
        }  
          
        static_cast<IndexedFaceSet *>(*it)->setCoordIndex(newCoordIndex);  
      } 
    }
    
    stateVariables->addNode(C);
  }
}

}
}
