#include "X3D_GLBuilderGroupingVisitor.h"
#include "X3D_Group.h"
#include "X3D_StaticGroup.h"
#include "X3D_Transform.h"
#include "X3D_Switch.h"
#include "GL_Group.h"
#include "GL_StaticGroup.h"
#include "GL_Transform.h"
#include "GL_Switch.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

GLBuilderGroupingVisitor::GLBuilderGroupingVisitor()
: GroupingVisitor()
{
  // Enter functions.
  define(Recorder<Group>::getEnterFunction(&GLBuilderGroupingVisitor::enterGroup));
  define(Recorder<StaticGroup>::getEnterFunction(&GLBuilderGroupingVisitor::enterStaticGroup));
  define(Recorder<Transform>::getEnterFunction(&GLBuilderGroupingVisitor::enterTransform));
  define(Recorder<Switch>::getEnterFunction(&GLBuilderGroupingVisitor::enterSwitch));
}

void GLBuilderGroupingVisitor::enterGroup(Group *G) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  GL::SFNode GG = stateVariables->getNode(G);
  if (GG == 0)
  {
    GG = new GL::Group();
    GG->setX3DReference(G);
    stateVariables->addCoupleNode(G, GG);
  }   
  stateVariables->pushNode(GG);
}

void GLBuilderGroupingVisitor::enterStaticGroup(StaticGroup *S) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  GL::SFNode GS = stateVariables->getNode(S);
  if (GS == 0)
  {
    GS = new GL::StaticGroup();
    GS->setX3DReference(S);
    stateVariables->addCoupleNode(S, GS);
  }   
  stateVariables->pushNode(GS);
}

void GLBuilderGroupingVisitor::enterTransform(Transform *T) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  GL::SFNode GT = stateVariables->getNode(T);
  if (GT == 0)
  {
    GT = new GL::Transform();
    GT->setX3DReference(T);
    stateVariables->addCoupleNode(T, GT);
  }   
  stateVariables->pushNode(GT);
}

void GLBuilderGroupingVisitor::enterSwitch(Switch *S) 
{
  GLBuilderStateVariables *stateVariables = Singleton<GLBuilderStateVariables>::getInstance();
  GL::SFNode GS = stateVariables->getNode(S);
  if (GS == 0)
  {
    GS = new GL::Switch();
    GS->setX3DReference(S);
    stateVariables->addCoupleNode(S, GS);
  }   
  stateVariables->pushNode(GS);
}

}
}
