#include "X3D_InlineLoaderNetworkingVisitor.h"
#include "X3D_Inline.h"
#include "X3D_Scene.h"

#include <iostream>

using namespace std;

namespace X3DTK {
namespace X3D {

InlineLoaderNetworkingVisitor::InlineLoaderNetworkingVisitor()
: NetworkingVisitor()
{
  // Enter functions.
  define(Recorder<Inline>::getEnterFunction(&InlineLoaderNetworkingVisitor::enterInline));
  // Leave functions.
  define(Recorder<Inline>::getLeaveFunction(&InlineLoaderNetworkingVisitor::leaveInline));
}

void InlineLoaderNetworkingVisitor::enterInline(Inline *I) 
{
  InlineLoaderStateVariables *stateVariables = Singleton<InlineLoaderStateVariables>::getInstance();
  X3DNode *F = stateVariables->getTop();
  X3DLoader *loader = stateVariables->getLoader();
  
  if ((stateVariables->isForced()) || (I->getLoad()))
  {
    Scene *s = loader->load(I->getUrl().front());
    if (s != 0)
    {
      MFNode children = s->getChildren();

      delete s;
          
      for (MFNode::const_iterator it = children.begin(); it != children.end(); ++it)
        F->addChild(*it);
    }
    else
      cx3d << "InlineLoaderNetworkingVisitor::enterInline: invalid url in Inline node!" << endl;

    F->removeChild(I);
  }
  
  stateVariables->addVisitedNode(I);  
  stateVariables->pushNode(I);
}

void InlineLoaderNetworkingVisitor::leaveInline(Inline *I) 
{
  InlineLoaderStateVariables *stateVariables = Singleton<InlineLoaderStateVariables>::getInstance();
  if (I->getParentList().empty())
    delete I;
  stateVariables->popNode();
}

}
}
